/*
   OpenGL SBalls

   File		: sballs.c
   Date		: 31/10/00
   Author	: Mustata Bogdan (LoneRunner)
   Contact	: lonerunner@planetquake.com
*/
#include <windows.h>
#include <io.h>
#include <wchar.h>
#include <stdio.h>
#include <process.h>
#include <math.h>
#include <stdlib.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>

#include "tga.h"

// Link with glu32.lib glut32.lib opengl32.lib

/* Some <math.h> files do not define M_PI... */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

typedef float vec_t;
typedef vec_t vec3_t[3];

enum {
    LEFT = 0,
    RIGHT,
};

int mButton = -1;
int mOldY, mOldX;
int speed = 1;

vec3_t eye = {0.0f, 0.0f, 6.0f};
vec3_t rot = {0.0f, 0.0f, 0.0f};
vec3_t rotm = {0.0f, 0.0f, 0.0f};

float LightAmbient[]=	{ 1.0f, 1.0f, 1.0f, 1.0f };
float LightDiffuse[]=	{ 1.0f, 1.0f, 1.0f, 1.0f };
float LightPosition[]=	{ 0.0f, 0.0f, 4.0f, 1.0f };

void glutResize(int width, int height)
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(55.0, (float)width/(float)height, 1.0, 300.0);
	glMatrixMode(GL_MODELVIEW);
}

void glutKeyboard(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 27:	exit(0);
		case 'f':
		case 'F':
			glutFullScreen();
			return;
		case 'a':
		case 'A':
			speed++;
			return;
		case 'z':
		case 'Z':
			speed--;
			return;
	}
}

void clamp(vec3_t v)
{
    int i;

    for (i = 0; i < 3; i ++)
        if (v[i] > 360 || v[i] < -360)
            v[i] = 0;
}

void glutMotion(int x, int y)
{
    if (mButton == LEFT)
    {
        rot[0] -= ((mOldY - y) * 180.0f) / 200.0f;
        rot[1] -= ((mOldX - x) * 180.0f) / 200.0f;
        clamp (rot);
    } else if (mButton == RIGHT)
    {
        eye[2] -= ((mOldY - y) * 180.0f) / 200.0f;
        clamp (rot);
    }

    mOldX = x;
    mOldY = y;
}

void glutMouse(int button, int state, int x, int y)
{
    if(state == GLUT_DOWN)
    {
        mOldX = x;
        mOldY = y;
        switch(button)
        {
            case GLUT_LEFT_BUTTON:
                mButton = LEFT;
            break;
            case GLUT_RIGHT_BUTTON:
                mButton = RIGHT;
            break;
        }
        } else if (state == GLUT_UP)
            mButton = -1;
}

void drawSphere(float x, float y, float z)
{
  int numMajor = 10; //24
  int numMinor = 10; //32
  float radius = 0.6;
  double majorStep = (M_PI / numMajor);
  double minorStep = (2.0 * M_PI / numMinor);
  int i, j;

  glPushMatrix();
  glTranslatef(x, y, z);

  glColor4f(1, 1, 1, 1);
  for (i = 0; i < numMajor; ++i)
  {
    double a = i * majorStep;
    double b = a + majorStep;
    double r0 = radius * sin(a);
    double r1 = radius * sin(b);
    GLfloat z0 = radius * cos(a);
    GLfloat z1 = radius * cos(b);

    glBegin(GL_TRIANGLE_STRIP);
    for (j = 0; j <= numMinor; ++j)
	{
      double c = j * minorStep;
      GLfloat x = cos(c);
      GLfloat y = sin(c);

      glNormal3f((x * r0) / radius, (y * r0) / radius, z0 / radius);
      glTexCoord2f(j / (GLfloat) numMinor, i / (GLfloat) numMajor);
      glVertex3f(x * r0, y * r0, z0);

      glNormal3f((x * r1) / radius, (y * r1) / radius, z1 / radius);
      glTexCoord2f(j / (GLfloat) numMinor, (i + 1) / (GLfloat) numMajor);
      glVertex3f(x * r1, y * r1, z1);
    }
    glEnd();
  }

  glPopMatrix();
}

void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();
	glTranslatef (-eye[0], -eye[1], -eye[2]);

    glColor3f(1, 1, 1);
	glBindTexture(GL_TEXTURE_2D, 2);
	glBegin(GL_QUAD_STRIP);
	glNormal3f(0, 0, 1); glTexCoord2f(0,0); glVertex3f(8, 4.1, -4);
	glNormal3f(0, 0, 1); glTexCoord2f(0,1); glVertex3f(8, -4.1, -4);
	glNormal3f(0, 0, 1); glTexCoord2f(1,0); glVertex3f(-8, 4.1, -4);
	glNormal3f(0, 0, 1); glTexCoord2f(1,1); glVertex3f(-8, -4.1, -4);
	glEnd();

	/* rotate the mouse */
	glRotatef(rot[0], 1.0f, 0.0f, 0.0f);
	glRotatef(rot[1], 0.0f, 1.0f, 0.0f);
	glRotatef(rot[2], 0.0f, 0.0f, 1.0f);

	/* rotate the balls */
	glRotatef(rotm[0], 1.0f, 0.0f, 0.0f);
	glRotatef(rotm[1], 0.0f, 1.0f, 0.0f);
	glRotatef(rotm[2], 0.0f, 0.0f, 1.0f);

	rotm[0] += speed;
	rotm[1] += -speed;
	rotm[2] += 0;

	glBindTexture(GL_TEXTURE_2D, 1);

	drawSphere(1, 1, 1);	drawSphere(-1, 1, 1);
	drawSphere(1, 1, -1);	drawSphere(-1, 1, -1);

	drawSphere(1, -1, 1);	drawSphere(-1, -1, 1);
	drawSphere(1, -1, -1);	drawSphere(-1, -1, -1);

	glPopMatrix();

	glFlush();
	glutSwapBuffers();
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(400, 300);
	glutInitWindowPosition(100, 100);
	glutCreateWindow("sballs");

	glutReshapeFunc(glutResize);
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutDisplay);
    glutKeyboardFunc(glutKeyboard);
	glutMouseFunc(glutMouse);
    glutMotionFunc(glutMotion);

	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);
	glEnable(GL_LIGHT1);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);

	loadTGA("face.tga", 1);
	loadTGA("back.tga", 2);

	glClearColor(0, 0, 0, 0);

	glutMainLoop();

	return 0;
}
